 
Data Access API
 
This API gives you access to LoadLeveler objects and allows you to
retrieve specific data from those objects. You can use this API to query
the negotiator daemon for information about its current set of jobs,
classes, and machines. This API can also be used to:
*  Query a LoadLeveler history file for job accounting information
*  Query the startd and schedd daemons for selected Workload Manager and
   job information
 
The Data Access API consists of the following subroutines:
*  ll_query
*  ll_set_request
*  ll_reset_request
*  ll_get_objs
*  ll_get_data
*  ll_next_obj
*  ll_free_objs
*  ll_deallocate
 
Using the data access API
 
To use this API, you need to call the data access subroutines in the
following order:
 
*  Call ll_query to initialize the query object.
*  Call ll_set_request to filter the objects you want to query.
*  Call ll_get_objs to retrieve a list of objects from a LoadLeveler
   daemon or history file.
*  Call ll_get_data to retrieve specific data from an object.
*  Call ll_next_obj to retrieve the next object in the list.
*  Call ll_free_objs to free the list of objects you received.
*  Call ll_deallocate to end the query.
 
To see code that uses these subroutines, refer to "Examples of using
the Data Access API" in  LoadLeveler: Using and Administering.  For more
information on LoadLeveler objects, see "Understanding the LoadLeveler
job object model" in LoadLeveler: Using and Administering.
 
Understanding the LoadLeveler job object model
 
The ll_get_data subroutine of the data access API allows you to access
the LoadLeveler job model. The LoadLeveler job model consists of objects
that have attributes and connections to other objects. An attribute is a
characteristic of the object and generally has a primitive data type
(such as integer, float, or character). The job name, submission time
and job priority are examples of attributes.
 
Objects are connected to one or more other objects through
relationships. An object can be connected to other objects through more
than one relationship, or through the same relationship. For example, A
Job object is connected to a Credential object and to Step objects
through two different relationships. A Job object can be connected to
more than one Step object through the same relationship of "having a
Step." When an object is connected through different relationships,
different specifications are used to retrieve the appropriate object.
 
When an object is connected to more than one object through the same
relationship, there are Count, GetFirst and GetNext specifications
associated with the relationship. The Count operation returns the number
of connections. You must use the GetFirst operation to initialize access
to the first such connected object. You must use the GetNext operation
to get the remaining objects in succession. You can not use GetNext
after the last object has been retrieved.
 
You can use the ll_get_data subroutine to access both attributes and
connected objects.   See the ll_get_data subroutine in LoadLeveler:
Using and Administering for more informaton.
 
The root of the job model is the Job object, as shown in  the
"LoadLeveler job object model" figure in the Data Access API in
LoadLeveler: Using and Administering. The job is queried for information
about the number of steps it contains and the time it was submitted. The
job is connected to a single Credential object and one or more Step
objects. Elements for these objects can be obtained from the job.
 
You can query the Credential object to obtain the ID and group of the
submitter of the job.
 
The Step object represents one executable unit of the job (all the tasks
that are executed together). It contains information about the execution
state of the step, messages generated during execution of the step, the
number of nodes in the step, the number of unique machines the step is
running on, the time the step was dispatched, the execution priority of
the step, the unique identifier given to the step by LoadLeveler, the
class of the step and the number of processes running for the step (task
instances). The Step is connected to one or more Switch Table objects,
one or more Machine objects and one or more Node objects. The list of
Machines represents all of the hosts where one or more nodes of the step
are running. If two or more nodes are running on the same host, the
Machine object for the host occurs only once in the step's Machine list.
The Step object is connected to one Switch Table object for each of the
protocols (MPI and LAPI) used by the Step.
 
Each Node object manages a set of executables that share common
requirements and preferences. The Node can be queried for the number of
tasks it manages, and is connected to one or more Task objects.
 
The Task object represents one or more copies of the same executable.
The Task object can be queried for the executable, the executable
arguments, and the number of instances of the executable.
 
The "Specifications for ll_get_data subroutine" table in the Data Access
API in LoadLeveler: Using and Administering describes the
specifications and elements available when you use the ll_get_data
subroutine.  Each specification name describes the object you need to
specify and the attribute returned. For example, the specification
LL_JobGetFirstStep includes the object you need to specify (LL_Job) and
the value returned (GetFirstStep).
 
This table is sorted alphabetically by object; within each object the
specifications are also sorted alphabetically.
 
ll_query subroutine
 
Purpose
 
The ll_query subroutine initializes the query object and defines the
type of query you want to perform. The LL_element created and the
corresponding data returned by this function is determined by the
query_type you select.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
LL_element * ll_query (enum QueryType query_type);
 
Parameters
 
query_type
   Can be:
   *  JOBS (to query job information)
   *  MACHINES (to query machine information)
   *  CLASSES (to query information about job classes)
   *  CLUSTER (to query cluster information)
   *  WLMSTAT (to query AIX Workload Manager)
   *  RESERVATIONS (to query reservation information)
 
Description
 
query_type is the input field for this subroutine.
 
This subroutine is used in conjunction with other data access
subroutines to query information about job and machine objects. You must
call ll_query prior to using the other data access subroutines.
 
Return values
 
This subroutine returns a pointer to an LL_element object. The pointer
is used by subsequent data access subroutine calls.
 
Error values
 
NULL     The subroutine was unable to create the appropriate pointer.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_reset_request, ll_get_objs,
ll_free_objs, ll_next_obj, ll_deallocate
 
ll_set_request subroutine
 
Purpose
 
The ll_set_request subroutine determines the data requested during a
subsequent ll_get_objs call to query specific objects. You can filter
your queries based on the query_type, object_filter, and data_filter you
select.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
int ll_set_request (LL_element *query_element, QueryFlags query_flags,
                    char **object_filter, DataFilter data_filter);
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query subroutine.
 
query_flags
   When query_type (in ll_query) is JOBS, query_flags can be the
   following:
 
   QUERY_ALL
         Query all jobs.
 
   QUERY_JOBID
         Query by job ID.
 
   QUERY_STEPID
         Query by step ID.
 
   QUERY_PROCID
         Query by process ID of a task of a job step.
 
   QUERY_USER
         Query by user ID.
 
   QUERY_GROUP
         Query by LoadLeveler group.
 
   QUERY_CLASS
         Query by LoadLeveler class.
 
   QUERY_HOST
         Query by machine name.
 
   QUERY_STARTDATE
         Query by job start dates. History file query only.
 
   QUERY_ENDDATE
         Query by job end dates. History file query only.
 
   QUERY_RESERVATION_ID
         Query job steps bound to a particular reservation.
 
   When query_type (in ll_query) is MACHINES, query_flags can be the
   following:
 
   QUERY_ALL
         Query all machines.
   QUERY_HOST
         Query by machine names.
 
   When query_type (in ll_query) is CLASSES, query_flags can be the
   following:
 
   QUERY_ALL
         Query all classes.
   QUERY_CLASS
         Query by LoadLeveler class.
 
   When query_type (in ll_query) is CLUSTER, query_flags can be the
   following:
 
   QUERY_ALL
      Query cluster information from central manager.
 
   When query_type (in ll_query) is WLMSTAT, query_flags can be the
   following:
 
   QUERY_STEPID
         Query by step ID.
 
   When query_type (in ll_query) is RESERVATIONS, query_flags can be the
   following:
 
   QUERY_ALL
         Query all reservations.
   QUERY_USER
         Query by user ID that owns the reservations.
   QUERY_GROUP
         Query by LoadLeveler group that owns the reservations.
   QUERY_HOST
         Query by machine name.
   QUERY_RESERVATION_ID
         Query by reservation ID
 
object_filter
   Specifies search criteria. The value you specify for object_filter is
   related to the value you specify for query_flags:
   *  If you specify QUERY_ALL, you do not need an object_filter.
   *  If you specify QUERY_JOBID, the object_filter must contain a list
      of job IDs (in the form host.jobid).
   *  If you specify QUERY_STEPID, the object_filter must contain a list
      of step IDs (in the form host.jobid.stepid).
   *  If you specify QUERY_PROCID, the object_filter must contain a list
      with a single process ID of a task of a job step.
   *  If you specify QUERY_USER, the object_filter must contain a list
      of user IDs.
   *  If you specify QUERY_CLASS, the object_filter must contain a list
      of LoadLeveler class names.
   *  If you specify QUERY_GROUP, the object_filter must contain a list
      of LoadLeveler group names.
   *  If you specify QUERY_HOST, the object_filter must contain a list
      of LoadLeveler machine names. When the query type is JOBS, the
      machine names must be the names of machines to which the jobs are
      submitted.
   *  If you specify QUERY_STARTDATE or QUERY_ENDDATE, the object filter
      must contain a list of two start dates or two end dates having the
      format MM/DD/YYYY.
   *  If you specify QUERY_RESERVATION_ID, the  object_filter must
      contain a list of reservation IDs.
 
     The last entry in the object_filter array must be NULL.
 
data_filter
   Filters the data returned from the object you query. The value you
   specify for data_filter is related to the value you specify for
   query_type:
   *  If you specify JOBS, data_filter can be ALL_DATA (the default),
      which returns the entire object, or Q_LINE, which returns the same
      information returned by the llq -f flag. For more information,
      see the description of the llq command in LoadLeveler: Using and
      Administering.
 
      Note: If you query a history file for job information, always
            specify ALL_DATA.
 
   *  If you specify JOBS and query_flags QUERY_PROCID you must always
      specify ALL_DATA.
   *  If you specify MACHINES, data_filter can be ALL_DATA (the default)
      which returns the entire object, or STATUS_LINE which returns the
      same information returned by the llstatus -f flag.  For more
      information, see the description of the llstatus command in
      LoadLeveler: Using and Administering.
   *  If you specify CLASSES, WLM_STAT, CLUSTER, or RESERVATIONS, then
      data_filter must be ALL_DATA (the default).
 
Description
 
query_element, query_flags, object_filter, and data_filter are the input
fields for this subroutine.
 
The QUERY_PROCID flag should not be used in combination with any other
query_flags.
 
You can request certain combinations of object filters by calling
ll_set_request more than once. When you do this, the query flags you
specify are or-ed together. The following are valid combinations of
object filters:
*  QUERY_JOBID and QUERY_STEPID: the result is the union of both queries
   and any other query flags (such as, QUERY_HOST) will be ignored.
*  QUERY_STARTDATE and QUERY_ENDDATE: the result is the intersection of
   both queries
*  QUERY_HOST, QUERY_USER, QUERY_GROUP, QUERY_CLASS, and
   QUERY_RESERVATION_ID: the result is the intersections of all of the
   queries
*  When the query_type is QUERY_RESERVATIONS, QUERY_RESERVATION_ID takes
   precedence and any other query flags are ignored (with the exception
   of QUERY_ALL, which always replaces any other query flags)
 
That is, to query jobs owned by certain users and on a specific
machines, issue ll_set_request first with QUERY_USER and the appropriate
user IDs, and then issue it again with QUERY_HOST and the appropriate
host names.
 
For example, suppose you issue ll_set_request with a user ID list of
anton and meg, and then issue it again with a host list of k10n10 and
k10n11. The objects returned are all of the jobs on k10n10 and k10n11
which belong to anton or meg.
 
Note that if you use two consecutive calls with the same flag, the
second call will replace the previous call.
 
Also, you should not use the QUERY_ALL flag in combination with any
other flag, since QUERY_ALL will replace any existing requests.
 
For history file queries, query_flags is restricted to the following:
QUERY_ALL, QUERY_STARTDATE, QUERY_ENDDATE.
 
Return values
 
This subroutine returns a zero to indicate success.
 
Error values
 
-1       You specified a query_element that is not valid.
-2       You specified a query_flag that is not valid.
-3       You specified an object_filter that is not valid.
-4       You specified a data_filter that is not valid.
-5       A system error occurred.
 
Related information
 
Subroutines: ll_get_data, ll_query, ll_reset_request, ll_get_objs,
ll_free_objs, ll_next_obj, ll_deallocate
 
ll_reset_request subroutine
 
Purpose
 
The ll_reset_request subroutine resets the request data to NULL for the
query_element you specify.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
int ll_reset_request (LL_element *query_element);
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query function.
 
Description
 
query_element is the input field for this subroutine.
 
This subroutine is used in conjunction with ll_set_request to change the
data requested with the ll_get_objs subroutine.
 
Return values
 
This subroutine returns a zero to indicate success.
 
Error values
 
-1       The subroutine was unable to reset the appropriate data.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_query, ll_get_objs,
ll_free_objs, ll_next_obj, ll_deallocate
 
ll_get_objs subroutine
 
Purpose
 
The ll_get_objs subroutine sends a query request to the daemon you
specify along with the request data you specified in the ll_set_request
subroutine. ll_get_objs receives a list of objects matching the request.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
LL_element * ll_get_objs (LL_element *query_element, LL_Daemon query_daemon,
                          char *hostname,int * number_of_objs,
                          int * error_code);			
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query function.
 
query_daemon
   Specifies the LoadLeveler daemon you want to query or whether you
   want to query job information stored in a history file. The enum
   LL_Daemon is defined in llapi.h as:
 
   enum LL_Daemon {LL_STARTD, LL_SCHEDD, LL_CM, LL_MASTER, LL_STARTER,
                   LL_HISTORY_FILE};
 
   The following indicates which daemons respond to which query flags.
   When query_type (in ll_query) is JOBS, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_ALL                        | negotiator (LL_CM), schedd       |
|                                  | (LL_SCHEDD), or history file     |
|                                  | (LL_HISTORY_FILE)                |
+----------------------------------+----------------------------------+
| QUERY_JOBID                      | negotiator (LL_CM) or schedd     |
|                                  | (LL_SCHEDD)                      |
+----------------------------------+----------------------------------+
| QUERY_STEPID                     | negotiator (LL_CM) or schedd     |
|                                  | (LL_SCHEDD)                      |
+----------------------------------+----------------------------------+
| QUERY_PROCID                     | startd (LL_STARTD)               |
+----------------------------------+----------------------------------+
| QUERY_USER                       | negotiator (LL_CM) or schedd     |
|                                  | (LL_SCHEDD)                      |
+----------------------------------+----------------------------------+
| QUERY_CLASS                      | negotiator (LL_CM) or schedd     |
|                                  | (LL_SCHEDD)                      |
+----------------------------------+----------------------------------+
| QUERY_HOST                       | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_STARTDATE                  | history file (LL_HISTORY_FILE)   |
+----------------------------------+----------------------------------+
| QUERY_ENDDATE                    | history file (LL_HISTORY_FILE)   |
+----------------------------------+----------------------------------+
| QUERY_RESERVATION_ID             | negotiator (LL_CM) or schedd     |
|                                  | (LL_SCHEDD)                      |
+----------------------------------+----------------------------------+
 
   When query_type (in ll_query) is MACHINES, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_ALL                        | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_HOST                       | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
 
   When query_type (in ll_query) is CLASSES, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_ALL                        | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_CLASS                      | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
 
   When query_type (in ll_query) is CLUSTER, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_ALL                        | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
 
   When query_type (in ll_query) is WLMSTAT, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_STEPID                     | startd (LL_STARTD)               |
+----------------------------------+----------------------------------+
 
   When query_type (in ll_query) is RESERVATIONS, the query_flags (in
   ll_set_request) listed in the left-hand column are responded to by
   the daemons listed in the right-hand column:
 
+----------------------------------+----------------------------------+
| QUERY_ALL                        | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_USER                       | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_GROUP                      | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_HOST                       | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
| QUERY_RESERVATION_ID             | negotiator (LL_CM)               |
+----------------------------------+----------------------------------+
 
hostname
   Specifies the hostname where the schedd or startd daemon is queried.
   If you specify NULL, the schedd daemon on the local machine is
   queried. To contact the negotiator daemon, you do not need to specify
   a hostname. If query_daemon is LL_HISTORY_FILE, hostname is the name
   of the history file.
 
number_of_objs
   Is a pointer to an integer representing the number of objects
   received from the daemon.
 
error_code
   Is a pointer to an integer representing the error code issued when
   the function returns a NULL value.  For more information, see the
   error value descriptions for this interface.
 
Description
 
query_element, query_daemon, and hostname are the input fields for this
subroutine. number_of_objs and error_code are output fields.
 
Each LoadLeveler daemon returns only the objects that it knows about.
 
Return values
 
This subroutine returns a pointer to the first object in the list. You
must use the ll_next_obj subroutine to access the next object in the
list.
 
Error values
 
This subroutine returns a NULL to indicate failure. The error_code
parameter is set to one of the following:
 
-1       query_element not valid.
-2       query_daemon not valid.
-3       Cannot resolve hostname.
-4       Request type for specified daemon not valid.
-5       System error.
-6       No valid objects meet the request.
-7       Configuration error.
-9       Connection to daemon failed.
-10      Error processing history file (LL_HISTORY_FILE query only).
-11      History file must be specified in the hostname argument
         (LL_HISTORY_FILE query only).
-12      Unable to access the history file (LL_HISTORY_FILE query only).
-13      DCE identity of calling program cannot be established.
-14      No DCE credentials.
-15      DCE credentials within 300 secs of expiration.
-16      64-bit API is not supported when DCE is enabled.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_query, ll_get_objs,
ll_free_objs, ll_next_obj, ll_deallocate
 
ll_get_data subroutine
 
Before you use this subroutine, make sure you are familiar with the
concepts discussed in the topic about the LoadLeveler job object model
in LoadLeveler: Using and Administering.
 
Purpose
 
The ll_get_data subroutine returns data from a valid LL_element.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
int ll_get_data (LL_element *element, enum LLAPI_Specification
                 specification, void* resulting_data_type);
 
Parameters
 
element
   Is a pointer to the LL_element returned by the ll_get_objs
   subroutine.
 
specification
   Specifies the data field within the data object you want to read.
 
resulting_data_type
   Is a pointer to the location where you want the data stored. If the
   call returns a nonzero value, an error has occurred and the contents
   of the location are undefined.
 
Description
 
object and specification are input fields, while resulting_data_type is
an output field.
 
The ll_get_data subroutine of the data access API allows you to access
LoadLeveler objects. The parameters of ll_get_data are a LoadLeveler
object (LL_element), a specification that indicates what information
about the object is being requested, and a pointer to the area where the
information being requested should be stored.
 
If the specification indicates an attribute of the element that is
passed in, the result pointer must be the address of a variable of the
appropriate type, and must be initialized to NULL. The type returned by
each specification is found in "Specifications for ll_get_data
subroutine" table in LoadLeveler: Using and Administering. If the
specification queries the connection to another object, the returned
value is of type LL_element. You can use a subsequent ll_get_data call
to query information about the new object.
 
The data type char* and any arrays of type int or char must be freed by
the caller.
 
LL_element pointers cannot be freed by the caller.
 
For the specifications, LL_MachineOperatingSystem and
LL_MachineArchitecture, resulting_data_type returns the string "???" if
a query is made before the associated records are updated with their
actual values by the appropriate startd daemons.
 
Return values
 
This subroutine returns a zero to indicate success.
 
Error values
 
-1       You specified an object that is not valid.
-2       You specified an LLAPI_Specification that is not valid.
 
Related information
 
Subroutines: ll_query, ll_set_request, ll_reset_request, ll_get_objs,
ll_next_obj, ll_free_objs, ll_deallocate
 
ll_next_obj subroutine
 
Purpose
 
The ll_next_obj subroutine returns the next object in the query_element
list you specify.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
LL_element * ll_next_obj (LL_element *query_element);
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query function.
 
Description
 
query_element is the input field for this subroutine.
 
Use this subroutine in conjunction with the ll_get_objs subroutine to
"loop" through the list of objects queried.
 
Return values
 
This subroutine returns a pointer to the next object in the list.
 
Error values
 
NULL     Indicates an error or the end of the list of objects.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_query, ll_get_objs,
ll_free_objs, ll_deallocate
 
ll_free_objs subroutine
 
Purpose
 
The ll_free_objs subroutine frees all of the LL_element objects in the
query_element list that were obtained by the ll_get_objs subroutine. You
must free the query_element by using the ll_deallocate subroutine.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
int ll_free_objs (LL_element *query_element);
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query function.
 
Description
 
query_element is the input field for this subroutine.
 
Return values
 
This subroutine returns a zero to indicate success.
 
Error values
 
-1       You specified a query_element that is not valid.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_query, ll_get_objs,
ll_reset_request, ll_free_objs
 
ll_deallocate subroutine
 
Purpose
 
The ll_deallocate subroutine deallocates the query_element allocated by
the ll_query subroutine.
 
Library
 
LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)
 
Syntax
 
#include "llapi.h"
 
int ll_deallocate (LL_element *query_element);
 
Parameters
 
query_element
   Is a pointer to the LL_element returned by the ll_query function.
 
Description
 
query_element is the input field for this subroutine.
 
Return values
 
This subroutine returns a zero to indicate success.
 
Error values
 
-1       You specified a query_element that is not valid.
 
Related information
 
Subroutines: ll_get_data, ll_set_request, ll_query, ll_get_objs,
ll_reset_request, ll_next_obj, ll_free_objs
