
Workload Management API

The Workload Management API consists of the following subroutines:

*  ll_control subroutine
*  ll_modify subroutine
*  ll_preempt subroutine
*  ll_preempt_jobs subroutine
*  ll_run_scheduler subroutine
*  ll_start_job subroutine
*  ll_terminate_job subroutine
*  ll_start_job_ext subroutine

The ll_control subroutine can be used to perform most of the LoadLeveler
control operations and is designed for general use.

The ll_preempt subroutine is not available in LoadLeveler for Linux. For
LoadLeveler 3.3, the ll_preempt subroutine was replaced  with the
ll_preempt_jobs subroutine.

The ll_start_job and ll_terminate_job subroutines are intended to be
used together with an external scheduler.

Note that the ll_start_job and ll_terminate_job subroutines
automatically connect to an alternate central manager if they cannot
contact the primary central manager.  You should use ll_start_job and
ll_terminate_job in conjunction with the query API.  The query API
collects information regarding which machines are available and which
jobs need to be scheduled.   See the "Query API" in LoadLeveler: Using
and Administering for more information.  

Note: The AIX Workload Manager (WLM) and the LoadLeveler Workload
      Management API are two distinct and unrelated components.

ll_control subroutine

Purpose

This subroutine allows an application program to perform most of the
functions that are currently available through the standalone commands:
llctl, llfavorjob, llfavoruser, llhold, and llprio.

In LoadLeveler for Linux only, ll_control returns an error condition
when:
*  DCE_ENABLEMENT is TRUE
*  SEC_ENABLEMENT is DCE or CTSEC
*  SCHEDULER_TYPE is GANG
*  NQS_DIR is specified

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_control (int control_version, enum LL_control_op control_op, 
                char **host_list, char ** user_list, char **job_list, 
                char **class_list, int priority);

Parameters

int control_version
   An integer indicating the version of ll_control being used. This
   argument should be set to LL_CONTROL_VERSION.

enum LL_control_op
   The control operation to be performed. The enum LL_control_op is
   defined in llapi.h as:
   enum LL_control_op {
   LL_CONTROL_RECYCLE, LL_CONTROL_RECONFIG, LL_CONTROL_START,LL_CONTROL_STOP,
   LL_CONTROL_DRAIN, LL_CONTROL_DRAIN_STARTD, LL_CONTROL_DRAIN_SCHEDD, 
   LL_CONTROL_PURGE_SCHEDD, LL_CONTROL_FLUSH, LL_CONTROL_SUSPEND,
   LL_CONTROL_RESUME, LL_CONTROL_RESUME_STARTD, LL_CONTROL_RESUME_SCHEDD,
   LL_CONTROL_FAVOR_JOB, LL_CONTROL_UNFAVOR_JOB, LL_CONTROL_FAVOR_USER,
   LL_CONTROL_UNFAVOR_USER, LL_CONTROL_HOLD_USER, LL_CONTROL_HOLD_SYSTEM,
   LL_CONTROL_HOLD_RELEASE, LL_CONTROL_PRIO_ABS, LL_CONTROL_PRIO_ADJ, 
   LL_CONTROL_START_DRAINED,}; 

char **host_list
   A NULL terminated array of host names.

char **user_list
   A NULL terminated array of user names.

char **job_list
   A NULL terminated array of job names. The job name that an element of
   job_list points to is a character string with one of the following
   formats: "host.jobid.stepid," "jobid.stepid," "jobid". host is the
   name of the machine to which the job was submitted (the default is
   the local machine), jobid is the job ID assigned to the job by
   LoadLeveler, and stepid is the job step ID assigned to a job step by
   LoadLeveler (the default is to include all the steps of a job).

char **class_list
   A NULL terminated array of class names.

int priority
   An integer representing the new absolute value of user priority or
   adjustment to the current user priority of job steps.

Description

The ll_control subroutine performs operations that are essentially
equivalent to those performed by the standalone commands: llctl,
llfavorjob, llfavoruser, llhold, and llprio. Because of this similarity,
descriptions of the ll_control operations are grouped according to the
standalone command they resemble.

llctl type of operations

These are the ll_control operations which mirror operations performed by
the llctl command. This summary includes a brief description of each of
the allowed llctl types of operations.  For more information, see the
llctl command description in LoadLeveler: Using and Administering.

LL_CONTROL_START: 
   Starts the LoadLeveler daemons on the specified machines. The calling
   program must have rsh privileges to start LoadLeveler daemons on
   remote machines.

   Note: LoadLeveler will fail to start if any value has been set for
         the MALLOCTYPE environment variable.

LL_CONTROL_START_DRAINED: 
   Starts the LoadLeveler in the drained state.

LL_CONTROL_STOP: 
   Stops the LoadLeveler daemons on the specified machines.

LL_CONTROL_RECYCLE:
   Stops, and then restarts, all of the LoadLeveler daemons on the
   specified machines.

LL_CONTROL_RECONFIG:
   Forces all of the LoadLeveler daemons on the specified machines to
   reread the configuration files.

LL_CONTROL_DRAIN:
   When this operation is selected, the following happens: (1) No
   LoadLeveler jobs can start running on the specified machines, and (2)
   No LoadLeveler jobs can be submitted to the specified machines.

LL_CONTROL_DRAIN_SCHEDD:
   No LoadLeveler jobs can be submitted to the specified machines.

LL_CONTROL_DRAIN_STARTD:
   Keeps LoadLeveler jobs from starting on the specified machines. If a
   class_list is specified, then the classes specified will be drained
   (made unavailable). The literal string "allclasses" can be used as an
   abbreviation for all of the classes.

LL_CONTROL_FLUSH:
   Terminates running jobs on the specified machines and send them back
   to the negotiator to await redispatch (if restart=yes).

LL_CONTROL_PURGE_SCHEDD:
   Purges the specified schedd host's job queue; a host_list consisting
   of one host name must be specified.

LL_CONTROL_SUSPEND:
   Suspends all jobs on the specified machines. This operation is not
   supported for parallel jobs.

LL_CONTROL_RESUME:
   Resumes job submission to, and job execution on, the specified
   machines.

LL_CONTROL_RESUME_STARTD:
   Resumes job execution on the specified machines; if a class_list is
   specified, then execution of jobs associated with these classes is
   resumed.

LL_CONTROL_RESUME_SCHEDD:
   Resumes job submission to the specified machines.

For these llctl type of operations, the user_list, job_list, and
priority arguments are not used and should be set to NULL or zero. The
class_list argument is meaningful only if the operation is
LL_CONTROL_DRAIN_STARTD, or LL_CONTROL_RESUME_STARTD. If class_list is
not being used, then it should be set to NULL. If host_list is NULL,
then the scope of the operation is all machines in the LoadLeveler
cluster. Unlike the standalone llctl command, where the scope of the
operation is either global or one host, ll_control operations allow the
user to specify a list of hosts (through the host_list argument). To
perform these operations, the calling program must have LoadLeveler
administrator authority. The only exception to this rule is the
LL_CONTROL_START operation.

llfavorjob type of operations

The llfavorjob type of control operations are: LL_CONTROL_FAVOR_JOB, and
LL_CONTROL_UNFAVOR_JOB. For these operations, the user_list, host_list,
class_list, and priority arguments are not used and should be set to
NULL or zero. LL_CONTROL_FAVOR_JOB is used to set specified job steps to
a higher system priority than all job steps that are not favored.
LL_CONTROL_UNFAVOR_JOB is used to unfavor previously favored job steps,
restoring the original priorities. The calling program must have
LoadLeveler administrator authority to perform these operations.

llfavoruser type of operations

The llfavoruser type of control operations are: LL_CONTROL_FAVOR_USER,
and LL_CONTROL_UNFAVOR_USER. For these operations, the host_list,
job_list, class_list, and priority arguments are not used and should be
set to NULL or zero. LL_CONTROL_FAVOR_USER sets jobs of one or more
users to the highest priority in the system, regardless of the current
setting. Jobs already running are not affected. LL_CONTROL_UNFAVOR_USER
is used to unfavor previously favored user's jobs, restoring the
original priorities. The calling program must have LoadLeveler
administrator authority to perform these operations.

llhold type of operations

The llhold type of control operations are: LL_CONTROL_HOLD_USER,
LL_CONTROL_HOLD_SYSTEM, and LL_CONTROL_HOLD_RELEASE. For these
operations, the class_list and priority arguments are not used, and
should be set to NULL or zero. LL_CONTROL_HOLD_USER and
LL_CONTROL_HOLD_SYSTEM place jobs in user hold and system hold,
respectively. LL_CONTROL_HOLD_RELEASE is used to release jobs from both
types of hold. The calling program must have LoadLeveler administrator
authority to put jobs into system hold, and to release jobs from system
hold. If a job is in both user and system holds then the
LL_CONTROL_HOLD_RELEASE operation must be performed twice to release the
job from both types of hold. If the user is not a LoadLeveler
administrator then the llhold types of operations have no effect on jobs
that do not belong to that user.

llprio type of operations

The llprio type of control operations are: LL_CONTROL_PRIO_ABS, and
LL_CONTROL_PRIO_ADJ. For these operations, the user_list, host_list, and
class_list arguments are not used, and should be set to NULL. llprio
type of operations change the user priority of one or more job steps in
the LoadLeveler queue. LL_CONTROL_PRIO_ABS specifies a new absolute
value of the user priority, and LL_CONTROL_PRIO_ADJ specifies an
adjustment to the current user priority. The valid range of LoadLeveler
user priorities is 0-100 (inclusive); 0 is the lowest possible priority,
and 100 is the highest. The llprio type of operations have no effect on
a running job step unless this job step returns to Idle state. If the
user is not a LoadLeveler administrator, then an llprio type of
operation has no effect on jobs that do not belong to that user.

Return values

0        The specified command has been sent to the appropriate
         LoadLeveler daemon.
-2       The specified command cannot be sent to the central manager.
-3       The specified command cannot be sent to one of the LoadL_master
         daemons.
-4       ll_control encountered an error while processing the
         administration or configuration file.
-6       A data transmission failure has occurred.
-7       The calling program does not have LoadLeveler administrator
         authority.
-19      An incorrect ll_control version has been specified.
-20      A system error has occurred.
-21      The system cannot allocate memory.
-22      A control_op operation that is not valid has been specified.
-23      The job_list argument contains one or more errors.
-24      The host_list argument contains one or more errors.
-25      The user_list argument contains one or more errors.
-26      Incompatible arguments have been specified for HOLD operation.
-27      Incompatible arguments have been specified for PRIORITY
         operation.
-28      Incompatible arguments have been specified for FAVORJOB
         operation.
-29      Incompatible arguments have been specified for FAVORUSER
         operation.
-30      An error occurred while ll_control tried to start a child
         process.
-31      An error occurred while ll_control tried to start the
         LoadL_master daemon.
-32      An error occurred while ll_control tried to execute the
         llpurgeschedd command.
-33      The class_list argument contains incompatible information.
-34      ll_control cannot create a file in the /tmp directory.
-35      LoadLeveler has encountered miscellaneous incompatible input
         specifications.
-36      DCE identity cannot be determined.
-37      No DCE credentials.
-38      DCE credentials within 300 secs of expiration.
-39      64-bit API not supported when DCE is enabled.
-40      This release of LoadLeveler for Linux does not support DCE.
-41      This release of LoadLeveler for Linux does not support CTSEC.
-42      This release of LoadLeveler for Linux does not support GANG.
-43      This release of LoadLeveler for Linux does not support PVM.
-44      This release of LoadLeveler for Linux does not support NQS.

Related information

Commands: llprio, llhold, llfavoruser, llfavorjob, llctl

ll_modify subroutine

Purpose

The ll_modify subroutine modifies the attributes of a submitted job
step.

In LoadLeveler for Linux only, ll_modify returns an error condition
when:
*  DCE_ENABLEMENT is TRUE
*  SEC_ENABLEMENT is DCE or CTSEC
*  SCHEDULER_TYPE is GANG
*  NQS_DIR is specified

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_modify (int version, void *errObj, LL_modify_param **param,
               char **steplist);

Parameters

version
   Input parameter that indicates the LoadLeveler API version (should
   have the same value as LL_API_VERSION in llapi.h).

errObj
   Provides the address of a pointer to LL_element that points to an
   error object if this function fails.

   The caller must free the error object storage before reusing the
   pointer. You can also use the ll_error subroutine to display error
   messages stored in the error object. If you are going to do so, the
   pointer should be initialized to NULL to avoid a segmentation fault
   when the pointer is passed to the ll_error subroutine.

param
   Provides the address of an array of 2 pointers to the LL_modify_param
   structure defined in llapi.h. The first pointer should point to an
   LL_modify_param structure already filled out by the caller. The
   second pointer should be assigned NULL. 	 

   In the LL_modify_param structure:    

   type     Describes the attribute to be modified.  

            On LoadLeveler for Linux type cannot be set to
            EXECUTION_FACTOR.

   data     Is a pointer to the new attribute value.    

   All job step attributes types that can be modified through 
   ll_modify( ) are listed in enum LL_modify_op in llapi.h.    

   The LL_modify_op structure stores user inputs to the function, where:    

   type     Is the type of the command option.

   data     Is a pointer to the data value associated with the command
            option.

   name     Is a NULL terminated array of job step names. Only a single
            job step is allowed in the current implementation.

steplist
   A NULL terminated array of step IDs. Only a single job step is
   allowed in the current implementation. Uses the following formats:
   "host.jobid.stepid," "jobid.stepid,". Where: 
   *  host - Is the name of the machine to which the job was submitted
      (the default is the local machine)
   *  jobid - Is the job ID assigned to the job by LoadLeveler
   *  stepid - Is the job step ID assigned to a job step by LoadLeveler

Description

ll_modify( ) is the API for the llmodify command. Refer to the llmodify
command for information about other available command options.

In enum LL_modify_op, the EXECUTION_FACTOR can be used with Gang
scheduling only. Only LoadLeveler administrators have authority to
modify this attribute of a job step.

In enum LL_modify_op, the system priority option will be ignored for any
job step not in an idle state. The data field for the SYSPRIO option is
an integer. The system priority for a job step set with the SYSPRIO
option will not be changed when LoadLeveler recalculates system
priorities.

Return values

The following return values are defined in llapi.h: 

MODIFY_SUCCESS
         Request successfully sent to LoadLeveler.

MODIFY_INVALID_PARAM
         An input parameter that is not valid was specified.

MODIFY_CONFIG_ERROR
         Errors encountered while processing config files.

MODIFY_NOT_IDLE
         Request failed, job step not in IDLE state.

MODIFY_WRONG_STATE
         Request failed, job step in wrong state.

MODIFY_NOT_AUTH
         Caller not authorized.

MODIFY_SYSTEM_ERROR
         LoadLeveler internal system error.

MODIFY_CANT_TRANSMIT
         Communication error while sending request.

MODIFY_CANT_CONNECT
         Failed to connect to LoadLeveler.

MODIFY_NO_DCE_SUPPORT_ERR
         DCE_ENABLEMENT was set to TRUE or SEC_ENABLEMENT was set to
         DCE. LoadLeveler for Linux does not support DCE.

MODIFY_NO_CTSEC_SUPPORT_ERR
         SEC_ENABLEMENT was set to CTSEC. LoadLeveler for Linux does not
         support CTSEC.

MODIFY_NO_GANG_SUPPORT_ERR
         SCHEDULER_TYPE was set to GANG. LoadLeveler for Linux does not
         support GANG scheduling.

MODIFY_NO_PVM_SUPPORT_ERR
         pvm_root was specified. LoadLeveler for Linux does not support
         PVM.

MODIFY_NO_NQS_SUPPORT_ERR
         NQS_DIR was specified. LoadLeveler for Linux does not support
         NQS.

MODIFY_OVERLAP_RESERVATION
         Request failed.  The requested change would cause the job step
         to overlap with a reservation.

Related information

Commands: llmodify command

Subroutines: ll_error

Example

/* mymodify.c - make a job step non-preemptable */
#include <stdio.h>
#include <string.h>
#include "llapi.h"

int main(int argc, char *argv[])
{
   int rc, exec_factor = 99;
   LL_modify_param mycmd, *cmdp[2];
   char *step_list[2];
   LL_element *errObj = NULL;
	char *errmsg;

if (argc < 2) {
   printf("Usage: %s job_step_name \n", argv[0]); exit(1);
}

step_list[0] = argv[1];
step_list[1] = NULL;
printf("\n*** Make Job Step %s non-preemptable ***\n\n",
        step_list[0]);

/* Initialize the LL_modify_param structure */
mycmd.type = EXECUTION_FACTOR;
mycmd.data = &exec_factor
cmdp[0] = &mycmd
cmdp[1] = NULL;

/* change execution factor to 99 for the job step */
printf("Change execution factor to %d\n", exec_factor);
rc = ll_modify(LL_API_VERSION, &errObj, cmdp, step_list);
if (rc) {
errmsg = ll_error(&errObj, 0);
printf("ll_modify() return code: %d\n%s\n", rc, errmsg);
free(errmsg);
      exit(1);
   }
   return 0;
}

ll_preempt subroutine

Purpose

The ll_preempt subroutine enables you to preempt a running job step or
to resume a job step that has already been preempted through the
llpreempt command or the ll_preempt subroutine (user-initiated). The
ll_preempt subroutine cannot resume a job step preempted through
PREEMPT_CLASS rules (system-initiated).

The ll_preempt subroutine is not available in LoadLeveler for Linux.

For LoadLeveler 3.3, the ll_preempt subroutine was replaced  with the
ll_preempt_jobs subroutine.

Library

LoadLeveler API library, libllapi.a

Syntax

#include "llapi.h"

int ll_preempt (int version, LL_element **errObj, char *job_step, 
                enum LL_preempt_op type);

Parameters

version
   Input parameter that indicates the LoadLeveler API version (should
   have the same value as LL_API_VERSION in llapi.h).

errObj
   Provides the address of a pointer to LL_element that points to an
   error object if this function fails.

   The caller must free the error object storage before reusing the
   pointer. You can also use the ll_error subroutine to display error
   messages stored in the error object. If you are going to do so, the
   pointer should be initialized to NULL to avoid a segmentation fault
   when the pointer is passed to the ll_error subroutine.

jobstep
   A string used to specify the name of a job step.

type   
   *  Preempts job step if type equals PREEMPT_STEP
   *  Resumes job step if type equals RESUME_STEP

Description

ll_preempt( ) is the API for the llpreempt command.
*  This function is for Gang scheduling and external schedulers
*  Only LoadLeveler administrators have authority to use this function

Return values

API_OK   Request successfully sent to LoadLeveler.

API_INVALID_INPUT
         An input parameter that is not valid was specified.

API_CONFIG_ERR
         Errors encountered while processing config files.

API_CANT_AUTH
         Caller not authorized.

API_CANT_CONNECT
         Failed to connect to LoadLeveler.

API_64BIT_DCE_ERR
         64-bit API not supported when DCE is enabled.

ll_preempt_jobs subroutine

Purpose

The ll_preempt_jobs subroutine is used to preempt a set of running job
steps using the specified preempt method, or to resume job steps that
have already been preempted with the preempt method of suspend through
the llpreempt command or the ll_preempt_jobs subroutine.  The
ll_preempt_jobs subroutine cannot resume a job step that was preempted
through the PREEMPT_CLASS rules, or a job step that was preempted with a
preempt method other than suspend.

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_preempt_jobs (int version, void *errObj, LL_preempt_param 
                     **param);

Parameters

version
   Is an input parameter that indicates the LoadLeveler API version
   (this should be the same value as LL_API_VERSION in llapi.h).

errObj
   Provides the address of a pointer to an LL_element that points to an
   error object if this function fails.

param
   Is a pointer to an array of structures.  The structure specifies the
   parameters for a preempt operation.  
   typedef struct LL_preempt_param {
        enum LL_preempt_op type;
        enum LL_preempt_method method;
        char ** user_list;
        char ** host_list;
        char ** jobstep_list;
   } LL_preempt_param;
   
   enum LL_preempt_op {PREEMPT_STEP, RESUME_STEP};
   
   enum LL_preempt_method {LL_PREEMPT_SUSPEND, LL_PREEMPT_VACATE,
        LL_PREEMPT_REMOVE, LL_PREEMPT_SYS_HOLD, LL_PREEMPT_USER_HOLD}    

   In the LL_preempt_param structure, the fields are defined as follows:    

   type              Is the type of operation on the job steps, preempt
                     or resume.

   method            Is the method to be used to preempt the specified
                     job steps.  This argument is ignored if the type
                     argument is not set to PREEMPT_STEP.  Valid values
                     for this argument are: 
                     *  LL_PREEMPT_SUSPEND
                     *  LL_PREEMPT_VACATE
                     *  LL_PREEMPT_REMOVE
                     *  LL_PREEMPT_SYS_HOLD
                     *  LL_PREEMPT_USER_HOLD                      

                     LoadLeveler for Linux does not support preemption
                     by the suspend method.  Job steps running on Linux
                     nodes will not be suspended when method in the
                     LL_preempt_param structure has the value
                     LL_PREEMPT_SUSPEND.  The call to the
                     ll_preempt_jobs subroutine is equivalent to a no-op
                     in this case.    

   In the enum LL_preempt_op structure, valid values are:  
   *  PREEMPT_STEP
   *  RESUME_STEP    

   In the enum LL_preempt_method structure, valid values are:    
   *  LL_PREEMPT_SUSPEND
   *  LL_PREEMPT_VACATE
   *  LL_PREEMPT_REMOVE
   *  LL_PREEMPT_SYS_HOLD
   *  LL_PREEMPT_USER_HOLD  

   user_list
            Is a pointer to a NULL-terminated array of pointers to
            strings containing  user names.  All running job steps
            belonging to all users in the list and monitored by the 
            machine the subroutine is running on will be preempted or
            resumed.  If a host_list is also specified, only the user's
            job steps monitored on the specified hosts will be
            preempted or resumed.

   host_list
            Is a pointer to a NULL-terminated array of pointers to
            strings containing host names.  All job steps monitored by
            the hosts will be preempted or resumed.  If a user_list is
            also provided, only the running job steps monitored by the
            hosts and owned by the specified users will be preempted or
            resumed.

   jobstep_list
            Is a NULL terminated array of pointers to strings containing
            the IDs of the job steps to be preempted or resumed.  The
            job step IDs are in the format of host.jobid.stepid where: 
            *  host - Is the name of machine to which the job was
               submitted. The default is the local machine.
            *  jobid - Is the job ID assigned to the job when it was
               submitted using the llsubmit command. The job ID is
               required.
            *  stepid - Is the step ID assigned to the job when it was
               submitted using the llsubmit command. The default is to
               include all the steps of the job. The jobstep_list is
               ignored if either a user_list or host_list is specified.

Description

The ll_preempt_jobs( ) subroutine is the API for the llpreempt command.
See the llpreempt command for information about the available command
options. In order to provide source code compatibility for applications
using the current version of the preempt function, the ll_preempt()
subroutine will not be modified to support the new preemption options. 
The ll_preempt() subroutine will continue to be supported as is.

Return values

0        Request successfully sent to the central manager.
-1       An LL_preempt_op that is not valid was specified.
-2       Cannot send request to central manager.
-3       An incorrect version was specified.
-4       Errors encountered while processing the LoadLeveler
         administration or configuration files.
-5       A system error occurred.
-6       A data transmission failure occurred.
-7       The calling program does not have LoadLeveler administrator
         authority.

ll_run_scheduler subroutine

Purpose

The ll_run_scheduler subroutine is used when the internal scheduling
interval has been disabled so that an external program can control when
the central manager attempts to schedule job steps. The ll_run_scheduler
subroutine sends a request to the central manager to run the scheduling
algorithm.

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_run_scheduler (int version);

Parameters

version
   Is an input parameter that indicates the LoadLeveler API version
   (this should be the same value as LL_API_VERSION in llapi.h).

Description

The ll_run_scheduler( ) subroutine sends a request to the central
manager to run the LoadLeveler scheduling algorithm.  The central
manager's scheduling algorithm will run only once each time the
llrunscheduler command is invoked.  Each time the scheduling algorithm
runs, the central manager will schedule as many job steps as the
current available resources allow.  The LoadLeveler scheduling interval
must be disabled, that is the configuration keyword NEGOTIATOR_INTERVAL
must be set to zero in order to use this algorithm.  Only LoadLeveler
administrators have authority to use this algorithm.

Return values

RUN_SCHED_SUCCESS
         Request successfully sent to the central manager.

RUN_SCHED_CONFIG_ERROR
         Errors were encountered while processing configuration files.

RUN_SCHED_NOT_AUTH
         Calling program does not have LoadLeveler administrator
         authority.

RUN_SCHED_NOT_AUTH
         An internal system error occurred.

RUN_SCHED_CANT_TRANSMIT
         A data transmission failure occurred.

RUN_SCHED_CANT_CONNECT
         The schedd cannot connect to the central manager.

ll_start_job subroutine

Purpose

This subroutine tells the LoadLeveler negotiator to start a job on the
specified nodes.

In LoadLeveler for Linux only, ll_start_job returns an error condition
when:
*  DCE_ENABLEMENT is TRUE
*  SEC_ENABLEMENT is DCE or CTSEC
*  SCHEDULER_TYPE is GANG
*  NQS_DIR is specified

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_start_job (LL_start_job_info *ptr);

Parameters

ptr
   Specifies the pointer to the LL_start_job_info structure that was
   allocated by the caller. The LL_start_job_info members are: 

   int version_num
      Represents the version number of the LL_start_job_info structure.
      Should be set to LL_PROC_VERSION.

   LL_STEP_ID StepId
      Represents the step ID of the job step to be started.

   char **nodeList
      Is a pointer to an array of node names where the job will be
      started. The first member of the array is the parallel master
      node. The array must be ended with a NULL.

Description

This subroutine does not allow you to specify adapter usage information.
Use the ll_start_job_ext subroutine instead.

You must set SCHEDULER_TYPE = API in the global configuration file to
use this subroutine.

Only jobs steps currently in the Idle state are started.

Only processes having the LoadLeveler administrator user ID can invoke
this subroutine.

An external scheduler uses this subroutine to start jobs that are in
idle state.  The list of jobs that are currently in the system is
retrieved with the ll_get_objs API function, passing in a query element
with type JOBS.  The list of machines available to run jobs on is
obtained with the ll_get_objs and a query element with type MACHINES. 
Additional data about both jobs and machines is obtained with the
ll_get_data function call.

When this function is used to start a step, adapter resources are
assigned to the step according to JCF network statements, if they are
present.  Adapter resources are assigned in the same manner as the
backfill scheduler assigns adapter resources, except that the
Communication Level on the network statement is ignored and a value of
AVERAGE is used.  It is the responsibility of the external scheduler to
ensure the machines to which the step is dispatched have sufficient
adapter resources to run the step.  Otherwise the step will not be
started.

Return values

This subroutine returns a value of zero to indicate the start job
request was accepted by the negotiator. However, a return code of zero
does not necessarily imply the job started. You can use the llq command
to verify the job started. Otherwise, this subroutine returns an integer
value defined in the llapi.h file.

Error values

-1       There is an error in the input parameter.
-2       The subroutine cannot connect to the central manager.
-4       An error occurred reading parameters from the administration or
         the configuration file.
-5       The negotiator cannot find the specified StepId in the
         negotiator job queue.
-6       A data transmission failure occurred.
-7       The subroutine cannot authorize the action because you are not
         a LoadLeveler administrator.
-8       The job object version number is incorrect.
-9       The StepId is not in the Idle state.
-10      One of the nodes specified is not available to run the job.
-11      One of the nodes specified does not have an available initiator
         for the class of the job.
-12      For one of the nodes specified, the requirements statement does
         not satisfy the job requirements.
-13      The number of nodes specified was less than the minimum or more
         than the maximum requested by the job.
-14      The LoadLeveler default scheduler is enabled.
-15      The same node was specified twice in ll_start_job nodeList.
-16      DCE identity cannot be determined.
-17      No DCE credentials.
-18      DCE credentials within 300 secs of expiration.
-19      64-bit not supported when DCE enabled.

Examples

Makefiles and examples which use this subroutine are located in the
samples/llsch subdirectory of the release directory. The examples
include the executable sch_api, which invokes the query API and the job
control API to start the second job in the list received from
ll_get_jobs on two nodes. You should submit at least two jobs prior to
running the sample. To compile sch_api, copy the sample to a writeable
directory and update the RELEASE_DIR field to represent the current
LoadLeveler release directory.

Related information

Subroutines: ll_get_jobs, ll_terminate_job, ll_get_nodes,
ll_start_jobs_ext.

ll_start_job_ext subroutine

Purpose

This subroutine tells the LoadLeveler negotiator to start a job on the
specified nodes, indicating which adapter and adapter resources to use.

In LoadLeveler for Linux only, ll_start_job_ext returns an error
condition when:
*  DCE_ENABLEMENT is TRUE
*  SEC_ENABLEMENT is DCE or CTSEC
*  SCHEDULER_TYPE is GANG
*  NQS_DIR is specified

An external scheduler uses this subroutine to start jobs that are in
idle state.  The list of jobs that are currently in the system is
retrieved with the ll_get_objs API function, passing in a query element
with type JOBS.  The list of machines available to run jobs on is
obtained with the ll_get_objs and a query element with type MACHINES. 
Additional data about both jobs and machines is obtained with the
ll_get_data function call.

When this function is used to start a step, the external scheduler
specifies the adapter resources that are assigned to the step and
network statements in the JCF, if they are present, are ignored. It
is the responsibility of the external scheduler to manage the
availability of adapter resources and LoadLeveler does not prevent or
detect the over commitment of adapter resources.

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_start_job_ext (LL_start_job_info_ext *ptr);

Parameters

ptr
   Specifies the pointer to the LL_start_job_info_ext structure that was
   allocated and populated by the caller. The LL_start_job_info_ext
   members are: 

   int version_num
      Represents the version number of the LL_start_job_info_ext
      structure. Should be set to LL_PROC_VERSION.

   LL_STEP_ID StepId
      Represents the step ID of the job step to be started.

   char ** nodelist
      A pointer to an array of node names where the job will be started.
      The first member of the array is the parallel master node. The
      array must be ended with a NULL.

   int adapterUsageCount
      This is the size of the adapterUsage list. To determine what this
      number should be, add all the adapter usages for all protocols
      needed by one task and multiply the result by the number of tasks
      in the job.

   LL_ADAPTER_USAGE * adapterUsage
      This is a list of adapter information. The size of this list is
      given by adapterUsageCount. The members of this structure are:

      char * dev_name
         The device name of the adapter to be used.

      char * protocol
         A character string representing the communication protocol this
         usage supports. Valid values are MPI, LAPI, and MPI_LAPI.

      char * subsystem
         The communication subsystem this usage supports. Valid values
         are IP or US.

      int wid
         For US subsystem usages, this indicates which adapter window ID
         to use. For IP subsystem usages, this field is ignored.

      uint64_t mem
         For US subsystem usages, this is the amount of adapter memory
         to dedicate to the adapter usage. For IP subsystem usages, this
         field is ignored.

Each element in the adapterUsage list represents one communication
channel for a task. If the subsystem is US (User Space), a communication
channel will require a switch adapter window. Adapter windows, and User
Space usages, must be specified on actual switch adapters that are only
accessible if AGGREGATE_ADAPTERS=False is specified in the configuration
file.

Description

You must set SCHEDULER_TYPE = API in the global configuration file to
use this subroutine. In order to have access to the physical switch
adapters in the LoadLeveler cluster (as opposed to virtual adapters
representing all of the adapters on a network or adapters striping
across multiple networks) you must specify AGGREGATE_ADAPTERS = False in
the global configuration file.

Only jobs steps currently in the Idle state are started.

Only processes having the LoadLeveler administrator user ID can invoke
this subroutine.

An external scheduler uses this subroutine in conjunction with the
ll_query and ll_get_data subroutines of the query API. The query API
returns information about which machines are available for scheduling
and which jobs are currently in the job queue waiting to be scheduled.

Return values

This routine returns a non-zero value to indicate the start request was
not delivered to the negotiator. These values are defined in the llapi.h
file and explained in "Error values." A return code of zero
indicates the request was successfully delivered to the negotiator, but
constraints on the negotiator may stop the job from starting. You can
use the llq command to verify the job started.

Error values

-1       There is an error in the input parameter.
-2       The subroutine cannot connect to the central manager.
-4       An error occurred reading parameters from the administration or
         the configuration file.
-5       The negotiator cannot find the specified StepId in the
         negotiator job queue.
-6       A data transmission failure occurred.
-7       The subroutine cannot authorize the action because you are not
         a LoadLeveler administrator.
-8       The job object version number is incorrect.
-9       The StepId is not in the Idle state.
-10      One of the nodes specified is not available to run the job.
-11      One of the nodes specified does not have an available initiator
         for the class of the job.
-12      For one of the nodes specified, the requirements statement does
         not satisfy the job requirements.
-13      The number of nodes specified was less than the minimum or more
         than the maximum requested by the job.
-14      The LoadLeveler default scheduler is enabled.
-15      The same node was specified twice in the ll_start_job nodeList.
-16      DCE identity cannot be determined.
-17      No DCE credentials.
-18      DCE credentials within 300 secs of expiration.
-19      64-bit API not supported when DCE is enabled.
-20      Adapter usage information does not match job structure.
-21      Adapter usage requested an adapter not on the machine.
-22      Wrong number of entries on adapter usage list.
-23      The adapter usage information did not specify the same protocol
         usage on each task.
-24      An invalid protocol string was specified on an adapter usage.
-25      The adapter usages specified incompatible protocols.
-26      An adapter usage specified a communication subsystem that was
         not IP or US.

Examples

Makefiles and examples which use this subroutine are located in the
samples/llsch subdirectory of the release directory. The examples
include the executable sch_api_ext, which invokes the query API and the
job control API to start the first job in the list received from
ll_query on one node and to cancel the second job in the list. To
compile sch_api_ext, copy the sample to a writeable directory and update
the RELEASE_DIR field to represent the current LoadLeveler release
directory.

Related information

Subroutines: ll_start_job, ll_query, ll_get_data, ll_terminate_job,
ll_start_job

ll_terminate_job subroutine

Purpose

This subroutine tells the negotiator to cancel the specified job step.

In LoadLeveler for Linux only, ll_terminate_job returns an error
condition when:
*  DCE_ENABLEMENT is TRUE
*  SEC_ENABLEMENT is DCE or CTSEC
*  SCHEDULER_TYPE is GANG
*  NQS_DIR is specified

Library

LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)

Syntax

#include "llapi.h"

int ll_terminate_job (LL_terminate_job_info *ptr);

Parameters

ptr
   Specifies the pointer to the LL_terminate_info structure that was
   allocated by the caller. The LL_terminate_job_info members are: 

   int version_num
      Represents the version number of the LL_terminate_job_info
      structure. Should be set to LL_PROC_VERSION.

   LL_STEP_ID StepId
      Represents the step ID of the job step to be terminated.

   char *msg
      A pointer to a null terminated array of characters. If this
      pointer is null or points to a null string, a default message is
      used. This message will be available through ll_get_data to tell
      the process why a program was terminated.

Description

You do not need to disable the default LoadLeveler scheduler in order to
use this subroutine.

Only processes having the LoadLeveler administrator user ID can invoke
this subroutine.

An external scheduler uses this subroutine in conjunction with the
ll_get_job subroutine (of the job control API) and ll_start_jobs
subroutine (of the query API). The external scheduler must use this
subroutine to return errors from ll_start_job to interactive parallel
jobs.

Return values

This subroutine returns a value of zero when successful, to indicate the
terminate job request was accepted by the negotiator. However, a return
code of zero does not necessarily imply the negotiator canceled the job.
Use the llq command to verify the job was canceled. Otherwise, this
subroutine returns an integer value defined in the llapi.h file.

Error values

-1       There is an error in the input parameter.
-4       An error occurred reading parameters from the administration or
         the configuration file.
-6       A data transmission failure occurred.
-7       The subroutine cannot authorize the action because you are not
         a LoadLeveler administrator or you are not the user who
         submitted the job.
-8       The job object version number is incorrect.
-17      No DCE credentials.
-18      DCE credentials within 300 secs of expiration.
-19      64-bit API not supported when DCE is enabled.

Examples

Makefiles and examples which use this subroutine are located in the
samples/llsch subdirectory of the release directory. The examples
include the executable sch_api, which invokes the query API and the job
control API to terminate the first job reported by the ll_get_jobs
subroutine. You should submit at least two jobs prior to running the
sample. To compile sch_api, copy the sample to a writeable directory and
update the RELEASE_DIR field to represent the current LoadLeveler
release directory.

Related information

Subroutines: ll_get_jobs, ll_start_job, ll_get_nodes
