
llchres - Change attributes of a reservation

Purpose

llchres -  Changes one or more of the attributes of a LoadLeveler
reservation. 

Syntax    

llchres { -? | -H | -v | [-q] [-t start_time | -t {+|-} minutes]
        [-d [+|-] duration] [-n [+|-] number_of_nodes |
        -h all | -h [+|-] host_list | -j job_step | -f job_command_file]
        [-U [+|-] user_list] [-G [+|-] group_list] [-s {yes|no}]
        [-i {yes|no}] [-u user] [-g group] -R reservation }

Flags
      
-?                
   Provides a short usage message.

-H 
   Provides extended help information.

-q 
   Specifies quiet mode: print no messages other than error messages.

-v               
   Displays the name of the command, release number, service level, 
   service level date, and lowest level of the operating system to run 
   this release.

-t start_time     
   Modifies the start_time for a reservation using a 24-hour clock. The 
   format [mm/dd[/[cc]yy]] HH:MM must be used.

-t {+|-} minutes  
   Modifies the start time for a reservation using minutes. When 
   minutes is preceded by a plus (+) sign or  minus (-) sign, the 
   current start time is postponed or moved closer, respectively, by
   the number of minutes specified.

-d [+|-] duration 
   Specifies a new duration for the reservation in minutes.  If 
   duration is preceded by a plus (+) sign or minus (-) sign, the 
   current duration of the reservation is increased or decreased,
   respectively, by the value specified.

-n [+|-] number_of_nodes
   Specifies a new request for the number of nodes to reserve. If 
   number_of_nodes is preceded by a plus (+) sign or  minus (-) sign, 
   the current number of nodes in the reservation is increased or 
   decreased, respectively, by the value specified.

-h all            
   Reserves all machines currently in the LoadLeveler cluster that can 
   be used for a reservation.

-h [+|-] host_list
   Specifies a change to the list of machines to be reserved.  When a 
   blank-delimited host list is specified, it indicates that a new list 
   of hosts are to be reserved. Specifying a plus (+) sign before host 
   list adds the listed machines to the reservation. Specifying a minus 
   (-) sign removes the listed machines from the reservation. Note that 
   when a host list is specified, the first character of any host name
   cannot be a plus (+) or minus (-) sign.

-j job_step       
   Specifies a new request that a set of nodes that the job step can run 
   on be reserved. The job step must be in an idle-like state and takes 
   the form [host.]jobid.stepid where: 
   *  host - The name of the schedd machine to which the job was 
      submitted. The default is the local machine.
   *  jobid - The job ID assigned to the job when it was submitted. The 
      job ID is required.
   *  stepid - The step ID assigned to the job when it was submitted. 
      The step ID is required.

   You must be an administrator or the owner of both the reservation and 
   job step to make this request. If the request to modify the 
   reservation is successful, the job step will be bound to the 
   reservation.

-f job_command_file
   Specifies the full path to a new job_command_file that will be 
   submitted and the first job step used to determine what nodes to
   reserve. The job ID of the newly created job will be displayed. All 
   job steps will be bound to the reservation, or if the modification
   request fails, will be placed in the NotQueued state. The job ID of 
   the newly created job will be displayed.

-U [+|-] user_list
   Specifies a new blank-delimited list of users who can use the 
   reservation. If the list of users is preceded by a plus (+) sign or  
   minus (-) sign, add those users to or remove those users from the 
   existing list of users that can use the reservation, respectively.

-G [+|-] group_list
   Specifies a new blank-delimited list of LoadLeveler groups that can 
   use the reservation. If the list of groups is preceded by a  plus (+)
   sign or minus (-) sign, those groups will be added to or removed from 
   the existing list of groups, respectively. The first character of any
   group name cannot be plus (+) sign or  minus (-) sign.

-s {yes|no}       
   SHARED mode is enabled when the reserved word yes is specified.  When 
   the reserved word no is specified, SHARED mode is disabled.

-i {yes|no}       
   REMOVE_ON_IDLE mode is enabled when the reserved word yes is 
   specified. When the reserved word no is specified, REMOVE_ON_IDLE 
   mode is disabled.

-u user           
   Specifies a new user ID that will own the reservation.

-g group          
   Specifies a new LoadLeveler group that will own the reservation.

-R reservation    
   Specifies the reservation ID to be modified. Input for reservation ID 
   takes the form [host.]rid[.r] where: 
   *  host - Is the name of the schedd machine to which the reservation 
      was submitted. The default is the local machine.
   *  rid - Is the ID assigned to the reservation when it was created. A 
      reservation ID is required.
   *  r - Indicates that this is a reservation ID (r is optional).       

Note: When a plus (+) sign or minus (-) sign is used to increase or 
decrease  a value, there cannot be spaces between the plus (+) sign or 
minus (-) sign and the value.

Description

The llchres command is for LoadLeveler administrators and the owner of a 
reservation.  Either all requested changes will be made and a message 
indicating the reservation request has been sent will be displayed, or 
none of the changes will be made and a message describing the reason for 
the failure will be displayed along with the message that the request 
was sent.  If a connection error occurs and the request cannot be sent, 
a message will be displayed.

Note that it is possible for a time out to occur while this command is 
waiting  for a response from the LoadLeveler central manager. Even if a 
time out occurs or the command process is killed, the command may still 
succeed.  To determine if the request has been granted, issue the llqres 
command.

Modification requests are subject to resource availability checks
and reservation policies.

Notes on changing a reservation:

*  Administrators can change the attributes of any reservation, 
   including the user ID that owns the reservation, while
   non-administrators can change attributes of only the
   reservations they own and cannot change the reservation owner.
*  A reservation owner who is not a LoadLeveler administrator cannot 
   change the start time, duration or reserved nodes if the start time 
   is not at least later than the current time by the number of minutes 
   specified by the RESERVATION_MIN_ADVANCE_TIME keyword.
*  The new reservation start time must be later than the current time 
   by at least the amount of time specified by the 
   RESERVATION_SETUP_TIME keyword.
*  A reservation in the CANCEL or COMPLETE state cannot be changed.
*  When a reservation is not in the WAITING state, the start time cannot 
   be changed.
*  When a reservation is not in the WAITING state, the only ways to 
   change reserved nodes are to add a number of nodes, or to add or 
   delete a list of nodes.
*  A reserved node with a bound step running cannot be removed from the 
   reservation.
*  When changing the reservation duration, the end time of the 
   reservation must be later than the current time.
*  You cannot delete all reserved nodes from a reservation; a 
   reservation must have at least one reserved node.
*  You cannot add a node that is already reserved to a reservation when
   using llchres -h +host_list.
*  You cannot delete a node that is not reserved from a reservation when  
   using llchres -h -host_list.
*  If you want to change the owner of a reservation, the new owner must 
   be able to own  an additional reservation (max_reservations for the 
   user is not specified or if specified, the quota is not used up yet).  
   If you want to change the group that owns the reservation, the new 
   group must be able to own an additional reservation (max_reservations 
   for the group is not specified  or if specified, the quota is not
   used up yet). If the change request is granted, the new owner and 
   group must have the permission to own a new reservation (they cannot 
   both have max_reservations unspecified).

This command is for the BACKFILL scheduler only.

Examples

1. To have reservation c94n16.20.r start an hour later than currently 
   scheduled with four fewer nodes, issue: 

   llchres -t +60 -n -4 -R c94n16.20.r

   You should receive a response similar to the following: 

   Request to change reservation c94n16.20.r has been sent to 
   LoadLeveler

2. To change the duration from 20 to 50 minutes and enable only users 
   chris, jay, and dave  to use the reservation c94n16.31.r, issue: 

   llchres -U chris jay dave -d 50 -R c94n16.31.r

   You should receive a response similar to the following: 

   Request to change reservation c94n16.31.r has been sent to 
   LoadLeveler

Security

LoadLeveler administrators and users can issue this command.
