
llsummary - Return job resource information for accounting

Purpose

Returns job resource information on completed jobs for accounting 
purposes.

You must enable the recording of accounting data in order to generate 
any of the four throughput reports. To do this, specify ACCT=A_ON in 
your LoadL_config file.  For detailed usage of the ACCT keyword, see 
the "Gathering job accounting data" topic in the "Configuring the 
LoadLeveler environment" chapter in LoadLeveler: Using and 
Administering.

Syntax
  
llsummary [-?] [-H] [-v] [-x] [-l] [-s <MM/DD/YYYY> to <MM/DD/YYYY>]
          [-e <MM/DD/YYYY> to <MM/DD/YYYY>] [-g <group>] 
	  [-G <unixgroup>]  [-a <allocated>]  [-r <report>]  
	  [-j <jobname>] [-d <section>] [-c <class>] [-u <user>] 
          [<filelist>]

Flags

-? 
   Provides a short usage message.

-H 
   Provides extended help information.

-v 
   Displays the name of the command, release number, service level, 
   service level date, and lowest level of the operating system to run 
   this release.

-x 
   Provides extended information. Using -x can produce a very long
   report. This option is meaningful only when used with the -l option.
   You must enable the recording of accounting data in order to collect 
   information with the -x flag. To do this, specify ACCT=A_ON A_DETAIL 
   in your LoadL_config file.

-l 
   Specifies that the long form of output is displayed.

-s 
   Specifies a range for the start date (queue date) for accounting 
   data to be included in this report. The format for entering the date
   is either MM/DD/YYYY (where MM is month, DD is day, and YYYY is year), 
   MM/DD/YY (where YY is a two-digit year value), or a string of digits 
   representing the number of seconds since 1970. If a two-digit year 
   value is used, then 69-99 maps to 1969-1999, and 00-68 maps to 
   2000-2068. The default is to include all the data in the report.

-e 
   Specifies a range for the end date (completion date) for
   accounting data to be included in this report. The format for
   entering the date is either MM/DD/YYYY (where MM is month, DD is day, 
   and YYYY is year), MM/DD/YY (where YY is a two-digit year value), or 
   a string of digits representing the number of seconds since 1970. The 
   default is to include all the data in the report.

-u user
   Specifies the user ID for whom accounting data is reported.

-c class
   Specifies the class for which accounting data is reported. For
   reports of all formats (short, long and extended),  llsummary
   will report information about every job which contains at least
   one step of the specified class. For the short format,
   llsummary also reports a job count and step count for each
   class; for these counts, a job's class is determined by the
   class of its first step.

-g group
   Specifies the LoadLeveler group for which accounting data is
   reported. For reports of all formats (short, long and
   extended),  llsummary reports information about every job which
   contains at least one step of the specified group. For the
   short format,  llsummary also reports a job count and step
   count for each group; for these counts, a job's group is
   determined by the group of its first step.

-G unixgroup
   Specifies the UNIX group for which accounting data is reported.

-a allocated
   Specifies the hostname that was allocated to run the job. You
   can specify the allocated host in short or long form.

-r report
   Specifies the report type. You must enable the recording of
   accounting data in order to collect information with the -r
   flag. To do this, specify ACCT=A_ON A_DETAIL in your
   LoadL_config file. You can choose one or more of the following
   reports:

   resource Provides CPU usage for all submitted jobs, including
            those that did not run. This is the default.

   avgthroughput
            Provides average queue time, run time, and CPU time
            for jobs that ran for at least some period of time.

   maxthroughput
            Provides maximum queue time, run time, and CPU time
            for jobs that ran for at least some period of time.

   minthroughput
            Provides minimum queue time, run time, and CPU time
            for jobs that ran for at least some period of time.

   throughput
            Selects all throughput reports.

   numeric  Reports CPU times in seconds rather than hours,
            minutes, and seconds

-d section
   Specifies the category (data section) for which you want to
   generate a report. You can specify one or more of the
   following: user, group, unixgroup, class, account, day, week,
   month, jobid, jobname, allocated.

-j host.jobid
   The job for which accounting data is reported. host is the name
   of the machine to which the job was submitted. The default is
   the local machine. jobid is the job ID assigned to the job when
   it was submitted using the llsubmit command. The entire
   host.jobid string is required.

filelist
   Is a blank-delimited list of files containing the accounting
   data. If not specified, the default is the local history file
   on the machine from which the command was issued. You can use
   the llacctmrg command to combine history files on different
   schedd machines into a single history file.

Related Information

In order to create an accounting report with the llsummary command, you 
must have read access to a history file. If a history file name is not 
specified as an argument, llsummary uses the history file in the 
LoadLeveler spool directory of the local machine as input. By default, 
the permissions of the spool directory are set by the llinit command to 
700 at install time. However, these permissions may be changed by a 
system administrators with root privileges.

The file permissions of the history file created by a LoadL_schedd
daemon are controlled by the HISTORY_PERMISSION configuration
keyword. A specification such as HISTORY_PERMISSION = rw-rw-r-- will
result in permission settings of 664. The default settings are 660.

Examples

The following example requests summary reports (standard listing)
of all the jobs submitted on your machine between the days of
January 12, 2005 and March 12, 2005:
     
llsummary -s 01/12/2005 to 03/12/2005

Results

The Standard Listing: The standard listing is generated when you
do not specify -l, -r, or -d with llsummary.
     
The standard listing includes the following fields: 

Name     User ID submitting jobs.

Class    Class specified or defaulted for the jobs.

Group    User's login group.

Account  Account number specified for the jobs.

Jobs     Count of the total number of jobs submitted by this user,
         class, group, or account.

Steps    Count of the total number of job steps submitted by this
         user, class, group, or account.

Job CPU  Total CPU time consumed by user's jobs.

Starter CPU
         Total CPU time consumed by LoadLeveler starter processes
         on behalf of the user jobs.

Leverage Ratio of job CPU to starter CPU.

The -r Listing: 

The -r listing includes the following fields: 

AvgQueueTime
         Average amount of time the job spent queued before
         running for this user, class, group, or account.

AvgRealTime
         Average amount of accumulated wall clock time for jobs
         associated with this user, class, group, or account.

AvgCPUTime
         Average amount of accumulated CPU time for jobs
         associated with this user, class, group, or account.

MinQueueTime
         Time of the job that spent the least amount of time in queue
         before running for this user, class, group, or account.

MinRealTime
         Time of the job with the least amount of wall clock time
         for this user, class, group, or account.

MinCPUime
         Time of the job with the least amount of CPU time for
         this user, class, group, or account.

The MaxQueueTime, MaxRealTime, and MaxCPUTime fields display the
time of the job with the greatest amount of queue, wall clock, and
CPU time, respectively. The ALL line for the Average listing
displays the average time for all users, classes, groups, and
accounts. The ALL line for the Minimum listing displays the time
of the job with the least amount of time for all users, classes,
groups, and accounts. The ALL line for the Maximum listing
displays the time of the job with the greatest amount of time for
all users, classes, groups, and accounts.

The Long Listing: For an explanation of fields in the long
listing, see the description of the output fields for the long
listing of the llq command. 

Note: Job statistics for the running hosts are available
      separately.   
             
Security

LoadLeveler administrators and users can issue this command.
