
llq - Query job status

Purpose

Returns information about job steps in the LoadLeveler queues.

Syntax
  
llq [-?] [-H] [-v] [-x] [-s] [ -l] [-w] [joblist] [-u userlist] 
    [-h hostlist] [-c classlist] [-R reservation_list] 
    [-f category_list] [-r category_list]

Flags

-? 
   Provides a short usage message.

-H 
   Provides extended help information.

-v 
   Displays the name of the command, release number, service level, 
   service level date, and lowest level of the operating system to run 
   this release.

-x 
   Provides extended information about the selected job. If the -x flag
   is used with the -r, -s, or -f flag, an error message is generated.  

   CPU usage and other resource consumption information on active
   jobs can only be reported using the -x flag if the LoadLeveler
   administrator has enabled it by specifying A_ON and A_DETAIL
   for the ACCT keyword in the LoadLeveler configuration file.

   Normally, llq connects with the central manager to obtain job
   information. When you specify -x, llq connects to the schedd
   machine that received the specified job to get extended job
   information. However, some statistics, including those corresponding
   to System Priority and q_sysprio, are available only from the central 
   manager. Do not use the -x option if you need these statistics.

   When specified without -l, CPU usage for active jobs is reported in 
   the short format.

   Note: Using both the -l and -x options without a joblist
         specification can produce a very long report and
         excessive network traffic.

-s 
   Provides information on why a selected list of jobs remain in the
   NotQueued, Idle or Deferred state. Along with this flag, users must
   specify a list of jobs. The user can also optionally supply a list of 
   machines to be considered when determining why the jobs cannot run. 
   If a list of machines is not provided, the default is the list of 
   machines in the LoadLeveler cluster. For each job, llq determines why 
   the job remains in one of the given states instead of Running.

-l 
   Specifies that a long listing be generated for each job for which 
   status is requested. Fields included in the long listing are shown in 
   "Results." 

   If -l is not specified, then the standard listing is generated as
   shown in "Results."

-w 
   Provides AIX Workload Manager (WLM) CPU and real memory statistics 
   for jobs in the running state. This flag can be used with a joblist, 
   steplist, or a single stepid. All other flags except -h will result 
   in an error message. 

   When the -w flag is augmented with a single stepid, the -h flag
   can be used in conjunction with -w to specify a single hostname.
         
   This flag can only be used when ENFORCE_CONSUMABLE_RESOURCES is
   enabled in the configuration file. Otherwise, an error message is 
   produced.           

   The following statistics are displayed for every node the job is
   running on:
   *  Current CPU resource consumption as a percentage of the
      total resources available
   *  Total CPU time consumed in milliseconds
   *  Current real memory consumption as a percentage of the total
      resources available
   *  The highest number of resident memory pages used

joblist
   Is a blank-delimited list of jobs of the form host.jobid.stepid
   where: 
   *  host is the name of the schedd machine to which the job was
      submitted (delimited by dot). The default is the local
      machine. 

      If the job was submitted from a submit-only machine, this is
      the name of the machine where the schedd daemon that sent
      the job to the negotiator resides.

   *  jobid is the job id assigned to the job when it was submitted
      using the llsubmit command.
   *  stepid (delimited by dot) Is the step id assigned to the job when
      it was submitted using the llsubmit command. The default is to 
      include all the job steps of the job associated with the jobid.

-u userlist
   Is a blank-delimited list of users. Only job steps belonging
   to users in this list are queried.

-h hostlist
   Is a blank-delimited list of machines. If the -s flag is not
   specified, only job steps monitored on machines in this list are
   queried. If the -s flag is specified, the list of machines is
   considered when determining why a job remains in the Idle state.  

   When the -h flag is used with the -w flag, only a single
   machine  name can be specified to obtain the WLM statistics for
   that machine.

-c classlist
   Is a blank-delimited list of classes. Only job steps belonging
   to classes in this list are queried.

-f category_list
   Is a blank-delimited list of categories you want to query. Each
   category you specify must be preceded by a percent sign. The
   category_list cannot contain duplicate entries. This flag
   allows you to create a customized version of the standard llq
   listing. You cannot use this flag with the -l flag. The output
   fields produced by this flag all have a fixed length. The
   output is displayed in the order in which you specify the
   categories. category_list can be one or more of the following:

   %a       Account number
   %c       Class
   %cc      Completion code
   %dc      Completion date
   %dd      Dispatch Date
   %dh      Hold date
   %dq      Queue date ("Submitted" date of "standard" llq output)
   %gl      LoadLeveler group
   %gu      UNIX group
   %h       Hostname (first hostname if more than one machine is
            allocated to the job step)
   %id      Step ID
   %is      Virtual image size
   %jn      Job name
   %jt      Job type
   %nh      Number of hosts allocated to the job step
   %o       Job owner
   %p       User priority
   %R       Reservation ID
   %sn      Step name
   %st      Status

-r category_list
   Is a blank-delimited list of formats (categories) you want to
   query. Each category you specify must be preceded by a percent
   sign. The category_list cannot contain duplicate entries. This
   flag allows you to create a customized version of the standard
   llq listing. You cannot use this flag with the -l flag. The
   output produced by this flag is considered raw, in that the
   fields can be variable in length. Output fields are separated
   by an exclamation point (!). The output is displayed in the
   order in which you specify the formats. category_list can be
   one or more of the formats listed under the -f flag.

-R reservation_list
   Is a blank-delimited list of reservation IDs. Only job steps
   bound to reservations in this list are queried. Input for
   the reservation IDs take the form [host.]rid[.r] where:  
   *  host - Is the name of the schedd machine to which the
      reservation was submitted. The default is the local machine.
   *  rid - Is the ID assigned to the reservation when it was
      created. A reservation ID is required.
   *  r - Indicates that this is a reservation ID (r is optional).

If a job step is not specified and if -u, -h, -c, or -R is not
specified, all jobs are queried.

If a job step is specified, you cannot specify -u, -h, -c, or -R, except 
in the cases of -w and -s, for which the -h flag has special meaning.

When -u, -h, -c, or -R are used in combination, the result is
the intersection of the job steps selected by each flag.

Description

The llq command returns information about job steps in the
LoadLeveler queues.

Examples

1. This example generates a long listing for job 8, job step 2
   submitted to machine gold: 

   llq -l gold.8.2

2. This example generates a standard listing for all job steps of
   job name 12 submitted to the local machine: 

   llq 12

3. This example generates a standard listing for all job steps
   owned  by either rich or nathan and bound to reservation 6: 

   llq -u rich nathan -R 6

4. This example generates an extended listing for all job steps of
   class batch or class highprio, being monitored on either c94n07
   or c94n09: 

   llq -x -c batch highprio -h c94n07 c94n09

5. The following example generates a standard listing for all job
   steps  bound to reservation c94n04.2.r: 

   llq -R c94n04.2.r

   You should receive a response similar to the following: 
          
   Id             Owner      Submitted   ST PRI Class        Running On 
   -------------- ---------- ----------- -- --- ------------ ----------
   c94n04.5.0     zhong       2/8  08:17 I  50  classA                  
         
   1 job step(s) in query, 1 waiting, 0 pending, 0 running, 0 held, 
   0 preempted

6. The following example generates a customized listing for all job 
   steps: 
          
   llq -f %id %o %R 

   You should receive a response similar to the following: 

   Step Id                  Owner       Reservation ID          
   ------------------------ ----------- ------------------------
   c94n04.5.0               zhong       c94n04.2.r              
   c94n04.4.0               zhong                               
         
   2 job step(s) in queue, 1 waiting, 0 pending, 0 running, 1 held, 
   0 preempted

Results

Standard listing: The standard listing is generated when you do not 
specify the -l option with the llq command.

The standard listing includes the following fields: 

Id       Job identifier presented in the format: host.jobid.stepid. If
         the llq command returns information about a job owned by a 
	 schedd in the same domain, then the domain of the hostname will 
	 not appear in the output. However, when the llq command reports 
	 information about a job owned by a schedd in a different 
	 domain, the fully qualified hostname is always included. Due to 
	 space limitations, the domain of the host may be truncated to 
	 fit in the space allocated to the Id field. If the domain is 
	 truncated, a dash (-) will appear at the end to indicate that 
	 characters have been left out. To see the full job ID, run llq 
	 with the -l flag.

Owner    Userid of the job submitter.

Submitted
         Date and time of job submission. 

ST       For more information, see the explanations of job
         states in LoadLeveler: Using and Administering.

PRI      User priority of the job step, where the values are defined 
         with the user_priority keyword in the job command file or 
	 changed by the llprio command, which is described in 
	 LoadLeveler: Using and Administering.

Class    Job class.

Running On
         If running, the name of the machine the job step is
         running on. This is blank when the job is not running.
         For a parallel job step, only the first machine is shown.

Customized, formatted standard listing: A customized and formatted
standard listing is generated when you specify llq with the -f flag.

Customized, unformatted standard listing: A customized and unformatted 
(raw) standard listing is generated when you specify llq with the -r  
flag. Output fields are separated by an exclamation point (!).

WLM CPU and real memory statistics listing: If the LoadLeveler
interface to AIX Workload Manager (WLM) is enabled, then the -w option
can be used to obtain CPU and real memory statistics of job steps in 
running state. Note that Large Page memory information is not included 
in the statistics since WLM does not manage Large Page memory.

The output listing associated with the -w option includes these fields:

Resource The resource being enforced by WLM. This is either CPU or
         Real Memory.

snapshot Current CPU or Real Memory consumption as a percentage of
         the total resources available.

total    Total CPU time consumed in milliseconds. CPU resource only.

high water
         The highest number of resident memory pages used. Real
         Memory resource only.

The long listing: The long listing is generated when you specify
the -l option with the llq command. This section contains sample
output for two llq commands: one querying a serial job and one
querying a parallel job. Following the sample output is an
explanation of all possible fields displayed by the llq command.

The long listing includes these fields:  

Job Step ID
         The job step identifier.

Job Name The name of the job.

Step Name
         The name of the job step

Structure Version
         An internal version identifier.

Owner    The userid of the user submitting the job.

Queue Date
         The date and time that LoadLeveler received the job.

Job Accounting Key
         The Job Accounting Key is a unique identifier for a
         LoadLeveler job step.  The accounting key is stored in
         the AIX accounting record for each process  associated
         with a LoadLeveler job step.  This field can be used to
         correlate AIX accounting records with  LoadLeveler
         accounting records.  The Job Accounting Key is stored in
         the history file and can be displayed  using the
         llsummary -l command.  

         This keyword is not applicable on LoadLeveler for Linux
         platforms.
               
         For more information on the Job Accounting Key, see the
         "Gathering job accounting data" topic in the the
         "Configuring the LoadLeveler environment" chapter in
         LoadLeveler: Using and Administering.

Status   The status (state) of the job. For more information,
         see the explanations of job states in LoadLeveler: Using
         and Administering.

Reservation ID
         The reservation ID that a job step is bound to. If a job step 
	 is not bound to any reservation, this field will be blank.

Requested Res. ID
         The reservation ID that a job step is requested to be
         bound to, but has not yet been bound to. This field will
         be set when a job submitted with a request to bind has
         been successfully submitted, but the bind has not yet
         occurred. The bind may never occur if either the owner of
         the job step is not  allowed to use the reservation, or
         if the reservation does not exist.  

         If a job command file is used to select nodes to reserve
         in a make  or change reservation request and the request
         fails, all steps of the job, if submitted successfully,
         will have MAKERES as their Requested Res. ID and the
         steps will be in the NQ state.

Execution Factor
         Used only for Gang scheduling, Execution factor is used
         to prevent preemption of a job step.

Dispatch Time
         The time the job was dispatched.

Completion Date
         Date and time job completed or exited.

Completion Code
         The status returned by the wait3 UNIX system call.

User Priority
         The priority of the job step, as specified by the user in
         the job command, or changed by the llprio command.

user_sysprio
         The user system priority of the job step, where the value
         is defined in the administration file.

class_sysprio
         The class priority of the job step, where the value is
         defined in the administration file.

group_sysprio
         The group priority of the job step, where the value is
         defined in the administration file.

System Priority
         The overall system priority of the job step, where the
         value is defined by the SYSPRIO expression in the
         configuration file.

q_sysprio
         The adjusted system priority of the job step. For more
         information, see the discussion of how a job's priority
         affects dispatching order in LoadLeveler: Using and
         Administering.

previous q_sysprio
         The previous adjusted system priority of the job step.  
         For more information, see the discussion of how a job's
         priority affects dispatching order in LoadLeveler: Using
         and Administering.

Notifications
         The notification status for the job step, where: 

         always
            Indicates notification is sent through the mail for
            all four notification categories below.

         complete
            Indicates notification is sent through the mail only
            when the job step completes.

         error
            Indicates notification is sent through the mail only
            when the job step terminates abnormally.

         never
            Indicates notification is never sent.

         start
            Indicates notification is sent through the mail only
            when starting or restarting the job step.

Virtual Image Size
         The value of the image_size keyword (if specified) or the
         size of the executable associated with the executable
         keyword (if specified) or the size of the job command file.

Large Page
         Indicates whether Large Page memory should be used to run
         this job step. Can be Y (use Large Page memory if
         available), N (No), or M (Mandatory).

Checkpointable
         Indicates if LoadLeveler considers the job step
         checkpointable (yes, no, or interval).

Ckpt Start Time
         The start time of the current checkpoint in progress.
         Blank if no checkpoint running.

Good Ckpt Time/Date
         Time and date stamp of the last successful checkpoint.

Ckpt Elapse Time
         Amount of time taken to perform the last successful
         checkpoint.

Fail Ckpt Time/Date
         Time and date stamp of the last failed checkpoint.

Ckpt Accum Time
         Accumulated time, in seconds, the job step has spent
         checkpointing.

Checkpoint File
         Location of the directory and file name to be used for
         checkpoint data.

Ckpt Execute Dir
         The directory where the job step's executable will be
         saved for checkpointable jobs.

Restart From Ckpt
         Indicates if a job has been restarted from an existing
         checkpoint (yes or no).

Restart Same Nodes
         Indicates if a job step should be restarted on the same
         nodes after vacate (yes or no).

Restart  Restart status (yes or no).

Preemptable
         Indicates whether a job step is preemptable (yes or no).

Preempt Wait Count
         Specifies the number of job steps that an idle job step
         must preempt before it can be started.

Hold Job Until
         Job step is deferred until this date and time.

Cmd      The name of the executable associated with the executable
         keyword (if specified) or the name of the job command file.

Args     Arguments that were passed to the executable.

Env      Environment variables to be set before executable runs.
         Appears only when the -x option is specified.

In       The file to be used for stdin.

Out      The file to be used for stdout.

Err      The file to be used for stderr.

Initial Working Dir
         The directory from which the job step is run. The relative 
	 directory from which the stdio files are accessed, if 
	 appropriate.

Dependency
         Job step dependencies as specified at job submission.

Resources
         Reflects the settings for the resources keyword in the
         job command file.

Requirements
         Job step requirements as specified at job submission.

Preferences
         Job step preferences as specified at job submission.

Task_geometry
         Reflects the settings for the task_geometry keyword in
         the job command file.

Blocking Reflects the settings for the blocking keyword in the job
         command file.

Step Type
         Type of job step:
         *  Serial
         *  General parallel

Min Processors
         The minimum number of processors needed for this job step.

Max Processors
         The maximum number of processors that can be used for
         this job step.

Allocated Hosts
         The machines that have been allocated for this job step.

Node Usage
         A request that a node be shared or not shared or that a
         time-slice is not shared. The user specifies this request
         while submitting the job.

Submitting Host
         The name of the machine to which the job is submitted.

Notify User
         The user to be notified by mail of a job's status.

Shell    The shell to be used when the job step runs.

LoadLeveler Group
         The LoadLeveler group associated with the job step.

Class    The class of the job step as specified at job submission.

Ckpt Hard Limit
         Checkpoint hard limit as specified at job step submission.

Ckpt Soft Limit
         Checkpoint soft limit as specified at job step submission.

Cpu Hard Limit
         CPU hard limit as specified at job submission.

Cpu Soft Limit
         CPU soft limit as specified at job submission.

Data Hard Limit
         Data hard limit as specified at job submission.

Data Soft Limit
         Data soft limit as specified at job submission.

Core Hard Limit
         Core hard limit as specified at job submission.

Core Soft Limit
         Core soft limit as specified at job submission.

File Hard Limit
         File hard limits as specified at job submission.

File Soft Limit
         File soft limit as specified at job submission.

Stack Hard Limit
         Stack hard limit as specified at job submission.

Stack Soft Limit
         Stack soft limit as specified at job submission.

Rss Hard Limit
         RSS hard limit as specified at job step submission.

Rss Soft Limit
         RSS soft limit as specified at job step submission.

Step Cpu Hard Limit
         Job step CPU hard limit as specified at job submission.

Step Cpu Soft Limit
         Job step CPU soft limit as specified at job submission.

Wall Clk Hard Limit
         Wall clock hard limit as specified at job submission.

Wall Clk Soft Limit
         Wall clock soft limit as specified at job submission.

Comment  The comment specified by the comment keyword in the job
         command file.

Account  The account number specified in the job command file.

Unix Group
         The effective UNIX group name.

NQS Submit Queue
         The name of the NQS pipe queue to which the NQS job will
         be routed.

NQS Query Queues
         The NQS queue names you can use to monitor the job.

DCE Principal
         The DCE principal name associated with the process that
         submitted the job to LoadLeveler.

User Space Windows
         The number of switch adapter windows assigned to the job step.

Negotiator Messages
         Informational messages for the job step if it is in the
         Idle or NotQueued state.

Bulk Transfer
         Indicates that the value will be Yes or No depending on
         whether the application requested that the communication
         subsystem use bulk transfer by specifying bulkxfer=yes in
         the job command file.

Adapter Requirement
         Reflects the settings of the network keyword in the job
         command file.

Step Cpus
         The total ConsumableCpus for the job step.

Step Virtual Memory
         The total ConsumableVirtualMemory for the job step.

Step Real Memory
         The total ConsumableMemory for the job step.

Step Adapter Memory
         The total adapter pinned memory for the job step.

When -x and -l options are specified, llq  also displays the
information listed below. If several LoadL_starter processes are
used for running this job step, then the values reported are
either cumulative totals or the maximum values. The same is true
for the processes of the job step.

For more information on the maxrss, ixrss, idrss, isrss, minflt,
majflt, nswap, inblock, oublock, msgsnd, msgrcv, nsignals, nvcsw,
and nivcsw fields, see the AIX and Linux documentation for the
rusage fields under the getrusage/getrusage64 subroutine.

Only minflt, majflt, and nswap are supported in LoadLeveler for Linux.

Running Host
         For a serial job step, the machine that is running this
         job step. For a parallel job step, the first machine that
         has been allocated for this job step.

Machine Speed
         For a serial job step, the value associated with the
         "speed" keyword of the machine that is running this job
         step. For a parallel job step, the value associated with
         the "speed" keyword of the first machine that has been
         allocated for this job step.

Starter User Time/Step User Time
         CPU user time of Starter/Step processes. Cumulative total.

Starter System Time/Step System Time
         CPU system time of Starter/Step processes. Cumulative total.

Starter Total Time/Step Total Time 
         CPU total time of Starter/Step processes. Cumulative total.

Starter maxrss/Step maxrss
         Maximum resident set size utilized. Maximum value.

Starter ixrss/Step ixrss
         An integral value indicating the amount of memory used by the
         text segment that was also shared among other processes 
	 (expressed in units of kilobytes * seconds-of-execution).

Starter idrss/Step Starter idrss
         An integral value of the amount of unshared memory in the
         data segment of a process (expressed in units of
         kilobytes * seconds-of-execution).

Starter isrss/Step isrss
         Depending on the Operating System, this field may contain
         the integral value of unshared stack size.

Starter minflt/Step minflt
         Number of page faults (reclaimed). Cumulative total.

Starter majflt/Step majflt
         Number of page faults (I/O required). Cumulative total.

Starter nswap/Step nswap
         Number of times swapped out. Cumulative total.

Starter inblock/Step inblock
         Number of times file system performed input. Cumulative total.

Starter oublock/Step oublock
         Number of times file system performed output. Cumulative total.

Starter msgsnd/Step msgsnd
         Number of IPC messages sent. Cumulative total.

Starter msgrcv/Step msgrcv
         Number of IPC messages received. Cumulative total.

Starter nsignals/Step nsignals
         Number of signals delivered. Cumulative total.

Starter nvcsw/Step nvcsw
         Number of context switches due to voluntarily giving up
         processor. Cumulative total.

Starter nivcsw/Step nivcsw
         Number of involuntary context switches. Cumulative total.

Other fields displayed for parallel jobs are: 

(Node) Name
         Blank value. Reserved for future use.

(Node) Requirements
         Job step requirements as specified at job submission.

(Node) Preferences
         Job step preferences as specified at job submission.

(Node) Node minimum
         Minimum number of machines of this Node type required to
         run this job step.

(Node) Node maximum
         Maximum number of machines of this Node type that can be
         used to run this job step.

(Node) Node actual
         Actual number of machines of this Node type that are used
         in the running of this job step.

(Node) Allocated Hosts
               
       * The machines of this Node type that have been
         allocated for this job step. The format is:
         hostname:task status:adapter usage, ... ,adapter usage + ... +
         hostname:task status:adapter usage, ... ,adapter usage

       * The adapter usage information has the format: adapter
         name(adapter window ID, network protocol, mode,
         adapter window memory)

(Node/Master Task) Executable
         The executable associated with the master task.

(Node/Master Task) Exec Args
         The arguments passed to the master task executable.

(Node/Master Task) Num Task Inst
         The number of task instances of the master task.

(Node/Master Task) Task Instance
               
       * Task instance information has the format:
         hostname:task ID:adapter usage, ... ,adapter usage
       * Adapter usage information has the format: adapter
         name(adapter window ID, network protocol, mode,
         adapter window memory)

(Node/Task) Num Task Inst
         The number of task instances.

(Node/Task) Task Instance
            
       * Task instance information has the format:
         hostname:task ID:adapter usage, ... ,adapter usage
       * Adapter usage information has the format: adapter
         name(adapter window ID, network protocol, mode,
         adapter window memory)

Security

LoadLeveler administrators and users can issue this command.
